package xyz.lychee.lagfixer.hooks;

import java.util.concurrent.TimeUnit;
import me.lucko.spark.api.Spark;
import me.lucko.spark.api.SparkProvider;
import me.lucko.spark.api.statistic.StatisticWindow;
import me.lucko.spark.api.statistic.StatisticWindow.CpuUsage;
import me.lucko.spark.api.statistic.StatisticWindow.MillisPerTick;
import me.lucko.spark.api.statistic.StatisticWindow.TicksPerSecond;
import me.lucko.spark.api.statistic.misc.DoubleAverageInfo;
import me.lucko.spark.api.statistic.types.DoubleStatistic;
import me.lucko.spark.api.statistic.types.GenericStatistic;
import org.bukkit.Bukkit;
import org.bukkit.scheduler.BukkitTask;
import xyz.lychee.lagfixer.LagFixer;
import xyz.lychee.lagfixer.managers.ErrorsManager;
import xyz.lychee.lagfixer.managers.HookManager;
import xyz.lychee.lagfixer.managers.SupportManager;
import xyz.lychee.lagfixer.objects.AbstractHook;
import xyz.lychee.lagfixer.objects.AbstractMonitor;

public class SparkHook extends AbstractHook {
   private BukkitTask task;

   public SparkHook(LagFixer plugin, HookManager manager) {
      super(plugin, manager);
   }

   public void load() {
      SupportManager.getInstance().setMonitor(new SparkMonitor());
      this.task = SupportManager.getInstance().getFork().runTimer(false, () -> {
         if (ErrorsManager.getInstance().isEnabled() && Bukkit.getOnlinePlayers().size() > 20) {
            Bukkit.dispatchCommand(Bukkit.getConsoleSender(), "spark profiler open");
         }

      }, 1L, 1L, TimeUnit.HOURS);
   }

   public void disable() {
      if (this.task != null) {
         this.task.cancel();
      }

   }

   public String plugin() {
      return "spark";
   }

   public BukkitTask getTask() {
      return this.task;
   }

   static class SparkMonitor extends AbstractMonitor {
      private final Spark spark = SparkProvider.get();

      protected double cpuProcess() {
         return this.spark.cpuProcess().poll(CpuUsage.SECONDS_10) * 100.0;
      }

      protected double cpuSystem() {
         return this.spark.cpuSystem().poll(CpuUsage.SECONDS_10) * 100.0;
      }

      protected double tps() {
         DoubleStatistic<StatisticWindow.TicksPerSecond> tps = this.spark.tps();
         return tps == null ? 20.0 : tps.poll(TicksPerSecond.SECONDS_10);
      }

      protected double tps(int time) {
         DoubleStatistic<StatisticWindow.TicksPerSecond> tps = this.spark.tps();
         return tps == null ? 20.0 : tps.poll()[time];
      }

      protected double mspt() {
         GenericStatistic<DoubleAverageInfo, StatisticWindow.MillisPerTick> mspt = this.spark.mspt();
         return mspt == null ? 0.0 : ((DoubleAverageInfo)mspt.poll(MillisPerTick.SECONDS_10)).median();
      }
   }
}
